import Acquisition

import DomCore

def wrap(node): """
    Helper function that wraps node up with DOM widget.  It uses
    acquisition 'magic' to place the node being wrapped in the
    acquisition context of the widget, so that the widget can access
    the node's data.
    
    You can extend this in a lot of ways. Instead of looking up in a
    dictionary based on nodeType only, you could look up on nodeName,
    like in XMLWidgets. Or, again like XMLWidgets, call a special
    isNodeSupported method on the widget to determine if the widget
    really 'wants' to support this particular node, based on node
    context or whatever the person who writes the widgets likes. So,
    'title' elements that are child nodes of 'section' elements may
    get different methods attached to them as 'title' elements that
    are child nodes of 'chapter' elements. (and therefore be rendered
    differently, for instance).

    Because the associations are made at runtime, you can vary how
    widgets will map to nodes as often as you like, too. In XMLWidgets
    I use this to produce an editor. In an editor, some nodes should
    be displayed in a special way (for instance the text should be in
    a HTML textarea so it can be edited). So you instruct the wrapping
    function to use the special 'edit' widget whenever the request
    is made to wrap that node.
    """
    return widget_dict[node.nodeType].__of__(node)

class ElementNodeWidget(Acquisition.Implicit):

    def wrap_id(self, id):
        """Wrap child node by id.
        """
        return wrap(getattr(self, id))
        
    def get_childNodes(self):
        """Get all child nodes of this node.
        """
        # wrap all children
        return map(self.wrap_id, self.childIds)

    def get_nodeType(self):
        """Get node type.
        """
        return self.nodeType
    
    def get_nodeName(self):
        """Get node name.
        """
        return self.nodeName

    def _node(self):
        """The actual node object is the acquisition parent of the
        widget (since the widget is wrapping the node by acquisition.
        """
        return self.aq_parent
    
    def get_parentNode(self):
        """Get the parent node of this node.
        """
        # wrap parent node (which is parent of the unwrapped node)
        return wrap(self._node().aq_parent)
    
    def appendChild(self, node):
        """Append a child to this node's list of child nodes.
        """
        self.addChildNode(node._node().aq_base)
    
class TextNodeWidget(Acquisition.Implicit):
    def text_content(self):
        """Get text content
        """
        return self.text

# nodeType to widget instance mapping used by wrap()
widget_dict = {
    DomCore.ELEMENT_NODE : ElementNodeWidget(),
    DomCore.TEXT_NODE : TextNodeWidget()
    }
    



